##############################################################
## MOD Title: ACL Locations
## MOD Author: Lord Le Brand < lordlebrand@hotmail.com > (Leroy Hellenbrand) N/A
## MOD Description: Adds possibility to create local permissions for other things
##					than forums. These can then be used alongside other local
##					permissions, even in info files for modules.
##
## MOD Version: 0.0.2
##
## Installation Level: Intermediate
## Installation Time: 15 Minutes
## Files To Edit: includes/auth.php,
##					includes/constants.php,
##					includes/functions_admin.php,
##					includes/functions_module.php,
##					includes/acp/auth.php
## Included Files: root/db_update.php
## License: http://opensource.org/licenses/gpl-license.php GNU General Public License v2
##############################################################
## For security purposes, please check: http://www.phpbb.com/mods/
## for the latest version of this MOD. Although MODs are checked
## before being allowed in the MODs Database there is no guarantee
## that there are no security problems within the MOD. No support
## will be given for MODs not found within the MODs Database which
## can be found at http://www.phpbb.com/mods/
##############################################################
## Author Notes:
##
##		HOW TO USE:
##
##		To add a new location, insert a new row into phpbb_acl_locations, and then enter the location_id as the second parameter for auth_admin::acl_add_option() (Only one location per function call is allowed). The columns for the phpbb_acl_locations table are as follows:
##
##			location_identifier:	Identifier string up to 10 chars (a-z, 0-9 and underscores allowed) for use in module info files. E.g. 'project' becomes $id_project in the info file. On forums table: ''
##			location_table:			The table constant for the table affected by the local permission. On forums table: 'FORUMS_TABLE'
##			location_primary:		The name of the primary key column in the location table. On forums table: 'forum_id'
##			location_title:			The name of the title column in the location table. On forums table: 'forum_name'
##			is_nestable:			Whether the location allows nesting. Will return padding for nestable locations in admin_auth. On forums table: 1
##
##		Modules:
##			Set $p_master->additional_ids for modules, before the $module->list_modules() method is called, like so:
##				$module->additional_ids = array($identifier => $value);
##			Wherein $identifier is the $identifier set in the locations table and $value is the value of the ID in question.
##
##############################################################
## MOD History:
##
##   2008-08-28 - Version 0.0.2
##		- Fixed 2 nubs in includes/functions_module.php
##		- Moved SQL insert and update to a separate PHP file
##		- Removed unnecessary CDATA blocks in MODX XML file
##
##   2008-07-31 - Version 0.0.1
##      - Added fallback for acl_location = 0
##      - Added support for auth_admin::acl_add_option()
##
##   2008-07-29 - Version 0.0.0
##      - First release
##
##############################################################
## Before Adding This MOD To Your Forum, You Should Back Up All Files Related To This MOD
##############################################################

#
#-----[ SQL ]------------------------------------------
#
ALTER TABLE phpbb_acl_options ADD acl_location tinyint(3) UNSIGNED NOT NULL DEFAULT '0';

CREATE TABLE phpbb_acl_locations (
	location_id tinyint(3) UNSIGNED NOT NULL auto_increment,
	location_identifier varchar(10) NOT NULL DEFAULT '',
	location_table varchar(40) NOT NULL DEFAULT '',
	location_primary varchar(40) NOT NULL DEFAULT '',
	location_title varchar(40) NOT NULL DEFAULT '',
	is_nestable tinyint(1) NOT NULL DEFAULT '0',
	PRIMARY KEY (location_id),
	KEY location_identifier (location_identifier)
) CHARACTER SET `utf8` COLLATE `utf8_bin`;

#
#-----[ OPEN ]------------------------------------------
#
includes/auth.php
#
#-----[ FIND ]------------------------------------------
#
			$sql = 'SELECT auth_option_id, auth_option, is_global, is_local
#
#-----[ IN-LINE FIND ]------------------------------------------
#
is_local
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
, acl_location
#
#-----[ FIND ]------------------------------------------
#
				$this->acl_options['option'][(int) $row['auth_option_id']] = $row['auth_option'];
#
#-----[ AFTER, ADD ]------------------------------------------
#
				$this->acl_options['location'][$row['auth_option']] = (int) $row['acl_location'];
#
#-----[ FIND ]------------------------------------------
#
				$sql = 'SELECT forum_id
					FROM ' . FORUMS_TABLE;
#
#-----[ BEFORE, ADD ]------------------------------------------
#
				if ($this->acl_options['location'][$opt])
				{
					$sql = 'SELECT location_table, location_primary
						FROM ' . ACL_LOCATIONS_TABLE . '
						WHERE location_id = ' . $this->acl_options['location'][$opt];

					$result = $db->sql_query($sql, 604800);
					$location_row = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);

					$location_primary	= $location_row['location_primary'];
					$location_table		= $location_row['location_table'];

					$sql = "SELECT $location_primary AS forum_id
						FROM " . constant($location_table);
				}
				else
				{

#
#-----[ AFTER, ADD ]------------------------------------------
#
				}
#
#-----[ OPEN ]------------------------------------------
#
includes/constants.php
#
#-----[ FIND ]------------------------------------------
#
// Additional tables
#
#-----[ AFTER, ADD ]------------------------------------------
#
define('ACL_LOCATIONS_TABLE', $table_prefix . 'acl_locations');
#
#-----[ OPEN ]------------------------------------------
#
includes/functions_admin.php
#
#-----[ FIND ]------------------------------------------
#
?>
#
#-----[ BEFORE, ADD ]------------------------------------------
#
/**
 * This is about the same as make_forum_select(false, false, false, true, false, false, true)
 * but for other ACL Locations.
 *
 * @param integer $location_id
 * @param boolean $padding
 * @return array
 */
function get_acl_location_array($location_id, $padding = false)
{
	global $db;

	$sql = 'SELECT *
		FROM ' . ACL_LOCATIONS_TABLE . '
		WHERE location_id = ' . (int) $location_id;
	$result = $db->sql_query($sql, 600);
	$location_row = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);

	$select_sql = ($location_row['is_nestable']) ? ', parent_id, left_id, right_id' : '';
	$order_sql = ($location_row['is_nestable']) ? 'ORDER BY left_id ASC' : '';

	$sql = "SELECT {$location_row['location_primary']} AS forum_id, {$location_row['location_title']} AS forum_name$select_sql
		FROM " . constant($location_row['location_table']) . "
		$order_sql";
	$result = $db->sql_query($sql, 600);

	$right = 0;
	$padding_store = array('0' => '');
	$padding = '';
	$location_list = array();

	while ($row = $db->sql_fetchrow($result))
	{
		if ($location_row['is_nestable'])
		{
			if ($row['left_id'] < $right)
			{
				$padding .= '&nbsp; &nbsp;';
				$padding_store[$row['parent_id']] = $padding;
			}
			else if ($row['left_id'] > $right + 1)
			{
				$padding = (isset($padding_store[$row['parent_id']])) ? $padding_store[$row['parent_id']] : '';
			}

			$right = $row['right_id'];
		}

		$forum_list[$row['forum_id']] = array_merge(array('padding' => $padding, 'selected' => false, 'disabled' => false), $row);
	}
	$db->sql_freeresult($result);
	unset($padding_store);

	return $location_list;
}
#
#-----[ OPEN ]------------------------------------------
#
includes/functions_module.php
#
#-----[ FIND ]------------------------------------------
#
	var $acl_forum_id = false;
#
#-----[ AFTER, ADD ]------------------------------------------
#
	var $additional_ids = array();
#
#-----[ FIND ]------------------------------------------
#
	function module_auth($module_auth, $forum_id = false)
#
#-----[ IN-LINE FIND ]------------------------------------------
#
$forum_id = false
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
, $additional_ids = false
#
#-----[ FIND ]------------------------------------------
#
					if (!preg_match('#(?:acl_([a-z0-9_]+)(,\$id)?)|(?:\$id)|(?:aclf_([a-z0-9_]+))|(?:cfg_([a-z0-9_]+))|(?:request_([a-zA-Z0-9_]+))#', $token))
#
#-----[ IN-LINE FIND ]------------------------------------------
#
,\$id
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
(_[a-z0-9_]{1,10})?
#
#-----[ IN-LINE FIND ]------------------------------------------
#
:\$id
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
(_[a-z0-9_]{1,10})?
#
#-----[ FIND ]------------------------------------------
#
		$forum_id = ($forum_id === false) ? $this->acl_forum_id : $forum_id;
#
#-----[ AFTER, ADD ]------------------------------------------
#
		if (!$additional_ids)
		{
			$additional_ids = $this->additional_ids;
		}

		foreach ($additional_ids as $key => $value)
		{
			$key = preg_replace(array('#[^a-z0-9_ .\\-\\+]#', '#[ .\\-\\+]#'), array('', '_'), strtolower($key));
			if (strlen($key) > 10)
			{
				$key = substr($key, 0, 10);
			}
			trim($key, '_');
			$id_{$key} = $value;
		}
#
#-----[ FIND ]------------------------------------------
#
		eval('$is_auth = (int) (' . preg_replace(array('#acl_([a-z0-9_]+)(,\$id)?#', '#\$id#', '#aclf_([a-z0-9_]+)#', '#cfg_([a-z0-9_]+)#', '#request_([a-zA-Z0-9_]+)#'), array('(int) $auth->acl_get(\'\\1\'\\2)', '(int) $forum_id', '(int) $auth->acl_getf_global(\'\\1\')', '(int) $config[\'\\1\']', '!empty($_REQUEST[\'\\1\'])'), $module_auth) . ');');
#
#-----[ IN-LINE FIND ]------------------------------------------
#
,\$id
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
(_[a-z0-9_]{1,10})?
#
#-----[ IN-LINE FIND ]------------------------------------------
#
, '#\$id#'
#
#-----[ IN-LINE BEFORE, ADD ]------------------------------------------
#
, '#\$id_([a-z0-9_]{1,10})#'
#
#-----[ IN-LINE FIND ]------------------------------------------
#
, '(int) $forum_id'
#
#-----[ IN-LINE BEFORE, ADD ]------------------------------------------
#
, '(int) $id_\\2'
#
#-----[ OPEN ]------------------------------------------
#
includes/acp/auth.php
#
#-----[ FIND ]------------------------------------------
#
			$sql = 'SELECT auth_option_id, auth_option, is_global, is_local
#
#-----[ IN-LINE FIND ]------------------------------------------
#
is_local
#
#-----[ IN-LINE AFTER, ADD ]------------------------------------------
#
, acl_location
#
#-----[ FIND ]------------------------------------------
#
				$this->acl_options['option'][(int) $row['auth_option_id']] = $row['auth_option'];
#
#-----[ AFTER, ADD ]------------------------------------------
#
				$this->acl_options['location'][$row['auth_option']] = (int) $row['acl_location'];
#
#-----[ FIND ]------------------------------------------
#
			$sql = 'SELECT forum_id
				FROM ' . FORUMS_TABLE;
#
#-----[ BEFORE, ADD ]------------------------------------------
#
			if ($this->acl_options['location'][$auth_option])
			{
				$sql = 'SELECT location_table, location_primary
					FROM ' . ACL_LOCATIONS_TABLE . '
					WHERE location_id = ' . $this->acl_options['location'][$auth_option];

				$result = $db->sql_query($sql);
				$location_row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);

				$location_primary	= $location_row['location_primary'];
				$location_table		= $location_row['location_table'];

				$sql = "SELECT $location_primary AS forum_id
					FROM " . constant($location_table);
			}
			else
			{
#
#-----[ AFTER, ADD ]------------------------------------------
#
			}
#
#-----[ FIND ]------------------------------------------
#
			$forum_names_ary = make_forum_select(false, false, true, false, false, false, true);
#
#-----[ BEFORE, ADD ]------------------------------------------
#
			if (!$this->acl_options['location'][$permission_type] || $this->acl_options['location'][$permission_type] == $this->acl_options['location']['f_'])
			{
#
#-----[ AFTER, ADD ]------------------------------------------
#
			}
			else
			{
				$forum_names_ary = get_acl_location_array($this->acl_options['location'][$permission_type]);
			}
#
#-----[ FIND ]------------------------------------------
#
		$sql = 'SELECT forum_id, forum_name
			FROM ' . FORUMS_TABLE . '
			WHERE ' . $db->sql_in_set('forum_id', array_keys($hold_ary)) . '
			ORDER BY left_id';
#
#-----[ BEFORE, ADD ]------------------------------------------
#
		if ($this->acl_options['location'][$auth_option])
		{
			$sql = 'SELECT location_table, location_primary, location_title
				FROM ' . ACL_LOCATIONS_TABLE . '
				WHERE location_id = ' . $this->acl_options['location'][$auth_option];

			$result = $db->sql_query($sql);
			$location_row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$sql = "SELECT {$location_row['location_primary']} AS forum_id, {$location_row['location_title']} AS forum_name
				FROM " . constant($location_row['location_table']) . '
			WHERE ' . $db->sql_in_set('forum_id', array_keys($hold_ary)) . '
			ORDER BY left_id';
		}
		else
		{
#
#-----[ AFTER, ADD ]------------------------------------------
#
		}
#
#-----[ FIND ]------------------------------------------
#
	function acl_add_option($options)
#
#-----[ IN-LINE FIND ]------------------------------------------
#
)
#
#-----[ IN-LINE BEFORE, ADD ]------------------------------------------
#
, $acl_location = false
#
#-----[ FIND ]------------------------------------------
#
			if ($row['is_local'])
			{
				$cur_options['local'][] = $row['auth_option'];
			}
		}
		$db->sql_freeresult($result);
#
#-----[ AFTER, ADD ]------------------------------------------
#
		if (!$acl_location)
		{
			$sql = 'SELECT location_id
				FROM ' . ACL_LOCATIONS_TABLE . '
				WHERE location_table = \'FORUMS_TABLE\'';

			$result = $db->sql_query($sql);
			$acl_location = $db->sql_fetchfield('location_id', false, $result);
			$db->sql_freeresult($result);
		}
#
#-----[ FIND ]------------------------------------------
#
					'is_local'		=> ($type == 'local' || $type == 'local_global') ? 1 : 0
#
#-----[ AFTER, ADD ]------------------------------------------
#
					, 'acl_location'	=> $acl_location
#
#-----[ DIY INSTRUCTIONS ]------------------------------------------
#
Run db_update.php and delete the file
Clear cache
#
#-----[ SAVE/CLOSE ALL FILES ]------------------------------------------
#
# EoM